import {makeStyles, MenuItem, MenuList, Text, tokens} from "@fluentui/react-components";
import {

    DocumentBulletListRegular, DocumentChevronDoubleRegular, DocumentCssRegular, DocumentDatabaseRegular,
    DocumentDismissRegular,
    DocumentImageRegular, DocumentJavascriptRegular, DocumentPdfRegular, DocumentYmlRegular,
    FolderRegular
} from "@fluentui/react-icons";
import {VirtualizerScrollView} from "@fluentui/react-components/unstable";
import React from "react";
import {useStoreBucket} from "../../store/bucket";
import { S3File} from "../../interfaces/connection";
import {useStoreFile} from "../../store/file";
import {useStoreConnection} from "../../store/connection";
import {TrimSuffix} from "../../hook/strings";

const useStyles = makeStyles({
    container: {
        marginTop: '0.5rem',
        maxWidth: 'calc(100vw - 25rem - 1px)',
        width: 'calc(100vw - 25rem - 1px)',
        height: 'calc(100vh - 9rem)',
    },
    row: {
        height: '32px',
        display: 'flex',
        marginLeft: '0.5rem',
        marginRight: '0.5rem',
    },
    item: {
        width: '100%',
        maxWidth: '100%',
        "&:hover": {
            color: tokens.colorNeutralForeground2BrandPressed,
        }
    },
    text: {
        overflow: 'hidden',
        width: 'calc(100vw - 32rem)',
        display: "block",
    },
    no_data: {
        flex: "1",
        height: '100%',
        width: '100%',
        display: 'flex',
        justifyContent: 'center',
        alignItems: 'center',
        fontSize: '8rem',
        flexDirection: 'column',
    },
})

export function ListFileComponent() {

    const styles = useStyles();
    const {conn_active} = useStoreConnection();
    const {bucket_active} = useStoreBucket()
    const {files_get, files_list} = useStoreFile()

    const filename = (key: string) => {
        let strs = TrimSuffix(key, "/").split("/")
        return strs[strs.length - 1]
    }

    async function handleClick(item: S3File) {
        if (item.type === 1) {
            files_get(conn_active!, bucket_active!, item.key)
            return
        }
    }

    function handleRightClick(e: React.MouseEvent<HTMLDivElement>, item: S3File) {
        e.preventDefault()
    }

    return <MenuList className={styles.container}>
        {files_list.length ?
            <VirtualizerScrollView
                numItems={files_list.length}
                itemSize={32}
                container={{role: 'list', style: {maxHeight: 'calc(100vh - 9rem)'}}}
            >
                {(idx) => {
                    return <div
                        className={styles.row} key={idx}
                        onClick={async () => {
                            await handleClick(files_list[idx])
                        }}
                        onContextMenu={async (e) => {
                            handleRightClick(e, files_list[idx])
                        }}>
                        <MenuItem className={styles.item}
                                  icon={files_list[idx].type ? <FolderRegular/> :
                                      <FileIcon name={files_list[idx].name}/>}>
                            <Text truncate wrap={false} className={styles.text}>
                                {filename(files_list[idx].key)}
                            </Text>
                        </MenuItem>
                    </div>
                }}
            </VirtualizerScrollView> : <div className={styles.no_data}>
                <div>
                    <DocumentDismissRegular/>
                </div>
                <Text size={900}>
                    没有文件
                </Text>
            </div>
        }
    </MenuList>
}

type FileIconProps = {
    name: string
}

function FileIcon(props: FileIconProps) {
    const strings = props.name.split(".")
    const suffix = strings[strings.length - 1]
    switch (suffix) {
        case "png":
            return <DocumentImageRegular/>
        case "jpg":
            return <DocumentImageRegular/>
        case "jpeg":
            return <DocumentImageRegular/>
        case "gif":
            return <DocumentImageRegular/>
        case "db":
            return <DocumentDatabaseRegular/>
        case "sqlite":
            return <DocumentDatabaseRegular/>
        case "sqlite3":
            return <DocumentDatabaseRegular/>
        case "pdf":
            return <DocumentPdfRegular/>
        case "css":
            return <DocumentCssRegular />
        case "js":
            return <DocumentJavascriptRegular/>
        case "yaml":
            return <DocumentYmlRegular/>
        case "yml":
            return <DocumentYmlRegular/>
        case "html":
            return <DocumentChevronDoubleRegular/>
        case "json":
            return <DocumentChevronDoubleRegular/>
        case "go":
            return <DocumentChevronDoubleRegular/>
        default:
            return <DocumentBulletListRegular/>
    }
}